@@ -1,7 +1,7 @@
 import requests
 import urllib.parse
 import json
-
+from bs4 import BeautifulSoup as bs
 
 class YoutubeSearch:
     def __init__(self, search_terms: str, max_results=None):
@@ -10,17 +10,21 @@ class YoutubeSearch:
 
         self.videos = self.search_videos()
         self.channels = self.search_channels()
-        
 
-    def channelInfo(id):
-        print("hi, searching for: {}".format(id))
+    def channelInfo(id, includeVideos=True):
+        headers = {"Accept-Language": "en-US,en;q=0.5"}
         encoded_search = urllib.parse.quote(id)
         BASE_URL = "https://youtube.com"
-        url = f"{BASE_URL}/channel/{encoded_search}/videos"
-        response = requests.get(url).text
+
+        if encoded_search[0:2] == "UC" and len(encoded_search) == 24:
+            url = f"{BASE_URL}/channel/{encoded_search}/videos"
+            response = requests.get(url, headers=headers).text
+        else:
+            url = f"{BASE_URL}/user/{encoded_search}/videos"
+            response = requests.get(url, headers=headers).text
 
         while 'window["ytInitialData"]' not in response:
-            response = requests.get(url).text
+            response = requests.get(url, headers=headers).text
 
         results = []
         start = (
@@ -31,59 +35,75 @@ class YoutubeSearch:
         end = response.index("};", start) + 1
         json_str = response[start:end]
         data = json.loads(json_str)
-        videoContent = data["contents"]["twoColumnBrowseResultsRenderer"]['tabs'][1]['tabRenderer']['content'][
-            'sectionListRenderer']['contents'][0]['itemSectionRenderer'][
-            'contents'][0]['gridRenderer']['items']
+
+        if includeVideos:
+            videoContent = data["contents"]["twoColumnBrowseResultsRenderer"]['tabs'][1]['tabRenderer']['content'][
+                'sectionListRenderer']['contents'][0]['itemSectionRenderer'][
+                'contents'][0]['gridRenderer']['items']
+                
         channelDetails = data["header"]['c4TabbedHeaderRenderer']
 
         try:
             sC = channelDetails['subscriberCountText']['simpleText'].split(" ")[0]
         except:
-            sC = "unknown"
+            try:
+                sC = channelDetails['subscriberCountText']['runs'][0]['text'].split(" ")[0]
+            except:
+                sC = "unavailable"
+            
         channel = {
             'id': id,
             'name': channelDetails['title'],
             'avatar': channelDetails['avatar']['thumbnails'][2]['url'],
             'subCount': sC
         }
-        
-        videos = []
-        for video in videoContent:
-            vid = {
-                'videoTitle': video['gridVideoRenderer']['title']['runs'][0]['text'],
-                'id': video['gridVideoRenderer']['videoId'],
-                'channelName': channelDetails['title'],
-                'timeStamp': video['gridVideoRenderer']['publishedTimeText']['simpleText'],
-                'views': video['gridVideoRenderer']['viewCountText']['simpleText'],
-                'videoThumb': video['gridVideoRenderer']['thumbnail']['thumbnails'][0]['url'],
-                'channelUrl': "/channel/{}".format(id)
-            }
-            videos.append(vid)
-
         results.append(channel)
-        results.append(videos)
+        
+        if includeVideos:
+            videos = []
+            for video in videoContent:
+
+                try:
+                    title=video['gridVideoRenderer']['title']['simpleText']
+                except:
+                    title=video['gridVideoRenderer']['title']['runs'][0]['text']
+
+                vid = {
+                    'id': video['gridVideoRenderer']['videoId'],
+                    'videoThumb': video['gridVideoRenderer']['thumbnail']['thumbnails'][1]['url'],
+                    'videoTitle': title,
+                    'channelName': channelDetails['title'],
+                    'channelId': id,
+                    'timeStamp': video['gridVideoRenderer']['publishedTimeText']['simpleText'],
+                    'views': video['gridVideoRenderer']['viewCountText']['simpleText'],
+                    'channelUrl': "/channel/{}".format(id)
+                }
+                videos.append(vid)
+            results.append(videos)
 
         return results
 
     def search_videos(self):
+        headers = {"Accept-Language": "en-US,en;q=0.5"}
         encoded_search = urllib.parse.quote(self.search_terms)
         BASE_URL = "https://youtube.com"
         url = f"{BASE_URL}/results?search_query={encoded_search}&lang=en"
-        response = requests.get(url).text
+        response = requests.get(url, headers=headers).text
         while 'window["ytInitialData"]' not in response:
-            response = requests.get(url).text
+            response = requests.get(url, headers=headers).text
         results = self.parse_html_videos(response)
         if self.max_results is not None and len(results) > self.max_results:
             return results[: self.max_results]
         return results
 
     def search_channels(self):
+        headers = {"Accept-Language": "en-US,en;q=0.5"}
         encoded_search = urllib.parse.quote(self.search_terms)
         BASE_URL = "https://youtube.com"
         url = f"{BASE_URL}/results?search_query={encoded_search}"
-        response = requests.get(url).text
+        response = requests.get(url, headers=headers).text
         while 'window["ytInitialData"]' not in response:
-            response = requests.get(url).text
+            response = requests.get(url, headers=headers).text
         results = self.parse_html_channels(response)
         if self.max_results is not None and len(results) > self.max_results:
             return results[: self.max_results]
@@ -164,12 +184,24 @@ class YoutubeSearch:
                     res["long_desc"] = video_data.get("descriptionSnippet", {}).get("runs", [{}])[0].get("text", None)
                     res["channel"] = video_data.get("longBylineText", {}).get("runs", [[{}]])[0].get("text", None)
                     res["duration"] = video_data.get("lengthText", {}).get("simpleText", 0)
-                    res["views"] = video_data.get("viewCountText", {}).get("simpleText", 0).split(" ")[0]
+                    
+                    try:
+                        res["views"] = video_data.get("viewCountText", {}).get("simpleText", 0).split(" ")[0]
+                    except:
+                        if "LIVE" in str(video_data.get("thumbnailOverlays")):
+                            res["views"] = "Livestream"
+                        else:
+                            res['views'] = "unavailable"
                     try:
                         res['publishedText'] = video_data.get("publishedTimeText", None).get("simpleText")
                     except:
-                        res['publishedText'] = "unavailable"
+                        if "UPCOMING" in str(video_data.get("thumbnailOverlays")):
+                            res['publishedText'] = "Scheduled"
+                        else:
+                            res['publishedText'] = "Unavailable"
+                            
                     res["url_suffix"] = video_data.get("navigationEndpoint", {}).get("commandMetadata", {}).get("webCommandMetadata", {}).get("url", None)
+                    res["channelId"] = video_data.get("longBylineText").get("runs")[0].get("navigationEndpoint").get("browseEndpoint").get("browseId")
                     results.append(res)
         return results
 
@@ -184,3 +216,4 @@ class YoutubeSearch:
 
     def channels_to_json(self):
         return json.dumps({"channels": self.channels})
+
